@@ -63,13 +63,16 @@
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/RustCrypto/media/8f1a9894/logo.svg",
     html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/media/8f1a9894/logo.svg",
-    html_root_url = "https://docs.rs/argon2/0.2.3"
+    html_root_url = "https://docs.rs/argon2/0.2.4"
 )]
 #![warn(rust_2018_idioms, missing_docs)]
 
 #[macro_use]
 extern crate alloc;
 
+#[cfg(feature = "std")]
+extern crate std;
+
 mod algorithm;
 mod block;
 mod error;
@@ -78,7 +81,12 @@ mod memory;
 mod params;
 mod version;
 
-pub use crate::{algorithm::Algorithm, error::Error, params::Params, version::Version};
+pub use crate::{
+    algorithm::Algorithm,
+    error::{Error, Result},
+    params::{Params, ParamsBuilder},
+    version::Version,
+};
 
 #[cfg(feature = "password-hash")]
 #[cfg_attr(docsrs, doc(cfg(feature = "password-hash")))]
@@ -93,10 +101,11 @@ use crate::{
     memory::{Memory, SYNC_POINTS},
 };
 use blake2::{digest, Blake2b, Digest};
+use core::convert::TryFrom;
 
 #[cfg(feature = "password-hash")]
 use {
-    core::convert::{TryFrom, TryInto},
+    core::convert::TryInto,
     password_hash::{Ident, Salt},
 };
 
@@ -225,7 +234,7 @@ impl<'key> Argon2<'key> {
         m_cost: u32,
         parallelism: u32,
         version: Version,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         let lanes = parallelism;
 
         if let Some(secret) = &secret {
@@ -290,7 +299,7 @@ impl<'key> Argon2<'key> {
         salt: &[u8],
         ad: &[u8],
         out: &mut [u8],
-    ) -> Result<(), Error> {
+    ) -> Result<()> {
         // TODO(tarcieri): move algorithm selection entirely to `Argon2::new`
         if self.algorithm.is_some() && Some(alg) != self.algorithm {
             return Err(Error::AlgorithmInvalid);
@@ -448,6 +457,31 @@ impl PasswordHasher for Argon2<'_> {
     }
 }
 
+impl<'key> TryFrom<Params> for Argon2<'key> {
+    type Error = Error;
+
+    fn try_from(params: Params) -> Result<Self> {
+        Argon2::try_from(&params)
+    }
+}
+
+impl<'key> TryFrom<&Params> for Argon2<'key> {
+    type Error = Error;
+
+    fn try_from(params: &Params) -> Result<Self> {
+        let mut argon2 = Argon2::new(
+            None,
+            params.t_cost,
+            params.m_cost,
+            params.p_cost,
+            params.version,
+        )?;
+
+        argon2.output_size = Some(params.output_size);
+        Ok(argon2)
+    }
+}
+
 #[cfg(all(test, feature = "password-hash"))]
 mod tests {
     use crate::{Argon2, Params, PasswordHasher, Salt, Version};
